@@ -6,6 +6,7 @@ from django import template
 from django.core.cache import cache
 from django.db.models import signals
 from django.utils.http import urlquote
+from django.utils.translation import get_language
 from django.template.defaulttags import url as url_tag
 
 from models import Tree, TreeItem
@@ -14,6 +15,77 @@ from models import Tree, TreeItem
 # Cache is only invalidated on sitetree or sitetree item change.
 CACHE_TIMEOUT = 31536000
 
+# Holds tree items processor callable or None.
+_ITEMS_PROCESSOR = None
+# Holds aliases of trees that support internationalization.
+_I18N_TREES = []
+
+
+def register_items_hook(callable):
+    """Registers a hook callable to process tree items right before they are passed to templates.
+
+    Callable should be able to:
+
+        a) handle ``tree_items`` and ``tree_sender`` key params.
+            ``tree_items`` will contain a list of extended TreeItem objects ready to pass to template.
+            ``tree_sender`` will contain navigation type identifier
+                (e.g.: `menu`, `sitetree`, `breadcrumbs`, `menu.children`, `sitetree.children`)
+
+        b) return a list of extended TreeItems objects to pass to template.
+
+
+    Example::
+
+        # Put the following code somewhere where it'd be triggered as expected. E.g. in app view.py.
+
+        # First import the register function.
+        from sitetree.sitetreeapp import register_items_hook
+
+        # The following function will be used as items processor.
+        def my_items_processor(tree_items, tree_sender):
+            # Suppose we want to process only menu child items.
+            if tree_sender == 'menu.children':
+                # Lets add 'Hooked: ' to resolved titles of every item.
+                for item in tree_items:
+                    item.title_resolved = 'Hooked: %s' % item.title_resolved
+            # Return items list mutated or not.
+            return tree_items
+
+        # And we register items processor.
+        register_items_hook(my_items_processor)
+
+    """
+    global _ITEMS_PROCESSOR
+    _ITEMS_PROCESSOR = callable
+
+
+def register_i18n_trees(aliases):
+    """Registers aliases of internationalized sitetrees.
+    Internationalized sitetrees are those, which are dubbed by other trees having
+    locale identifying suffixes in their aliases.
+
+    Lets suppose ``my_tree`` is the alias of a generic tree. This tree is the one
+    that we call by its alias in templates, and it is the one which is used
+    if no i18n version of that tree is found.
+
+    Given that ``my_tree_en``, ``my_tree_ru`` and other ``my_tree_{locale-id}``-like
+    trees are considered internationalization sitetrees. These are used (if available)
+    in accordance with current locale used by project.
+
+    Example::
+
+        # Put the following code somewhere where it'd be triggered as expected. E.g. in main urls.py.
+
+        # First import the register function.
+        from sitetree.sitetreeapp import register_i18n_trees
+
+        # At last we register i18n trees.
+        register_i18n_trees(['my_tree', 'my_another_tree'])
+
+    """
+    global _I18N_TREES
+    _I18N_TREES = aliases
+
 
 class SiteTree(object):
 
@@ -33,7 +105,7 @@ class SiteTree(object):
         cache_ = cache.get('sitetrees')
         if cache_ is None:
             # Init cache dictionary with predefined enties.
-            cache_ = {'sitetrees': {}, 'urls': {}, 'parents': {}, 'items_by_ids': {}}
+            cache_ = {'sitetrees': {}, 'urls': {}, 'parents': {}, 'items_by_ids': {}, 'tree_aliases': {}}
         self.cache = cache_
 
     def cache_save(self):
@@ -44,6 +116,7 @@ class SiteTree(object):
         """Empties cached sitetree data."""
         self.cache = False
         cache.delete('sitetrees')
+        cache.delete('tree_aliases')
 
     def get_cache_entry(self, entry_name, key):
         """Returns cache entry parameter value by its name."""
@@ -67,6 +140,22 @@ class SiteTree(object):
         if not self.global_context or hash(context) != hash(self.global_context):
             self.global_context = context
 
+    def resolve_tree_i18_alias(self, alias):
+        """Resolves internationalized tree alias.
+        Verifies whether a separate sitetree is available for currently active language.
+        If so, returns i18n alias. If not, returns the initial alias.
+        """
+        if alias in _I18N_TREES:
+            current_language_code = get_language().replace('_', '-').split('-')[0]
+            i18n_tree_alias = '%s_%s' % (alias, current_language_code)
+            trees_count = self.get_cache_entry('tree_aliases', i18n_tree_alias)
+            if trees_count is False:
+                trees_count = Tree.objects.filter(alias=i18n_tree_alias).count()
+                self.set_cache_entry('tree_aliases', i18n_tree_alias, trees_count)
+            if trees_count:
+                alias = i18n_tree_alias
+        return alias
+
     def get_sitetree(self, alias):
         """Gets site tree items from the given site tree.
         Caches result to dictionary.
@@ -75,6 +164,7 @@ class SiteTree(object):
         """
         self.cache_init()
         sitetree_needs_caching = False
+        alias = self.resolve_tree_i18_alias(alias)
         sitetree = self.get_cache_entry('sitetrees', alias)
         if not sitetree:
             sitetree = TreeItem.objects.select_related('parent', 'tree').\
@@ -211,7 +301,7 @@ class SiteTree(object):
                 if '-' in argument or '_':
                     argument = '"%s"' % argument
                 view_arguments.append(argument)
-            
+
             view_arguments = ' '.join(view_arguments).strip()
             view_path = view_path[0]
             url_pattern = u'%s %s' % (view_path, view_arguments)
@@ -324,7 +414,17 @@ class SiteTree(object):
 
         # Parse titles for variables.
         menu_items = self.parse_titles(menu_items, context)
-        return menu_items
+        return self.apply_hook(menu_items, 'menu')
+
+    def apply_hook(self, items, sender):
+        """Applies item processing hook, registered with ``register_item_hook()``
+        to items supplied, and returns processed list.
+        Returns initial items list if no hook is registered.
+
+        """
+        if _ITEMS_PROCESSOR is None:
+            return items
+        return _ITEMS_PROCESSOR(tree_items=items, tree_sender=sender)
 
     def check_access(self, item, context):
         """Checks whether user have access to certain item."""
@@ -354,7 +454,7 @@ class SiteTree(object):
         if current_item is not None:
             self.breadcrumbs_climber(tree_alias, current_item)
             self.cache_breadcrumbs.reverse()
-        return self.cache_breadcrumbs
+        return self.apply_hook(self.cache_breadcrumbs, 'breadcrumbs')
 
     def tree(self, tree_alias, context):
         """Builds and returns tree structure for 'sitetree_tree' tag."""
@@ -362,8 +462,8 @@ class SiteTree(object):
         # No items in tree, fail silently.
         if not sitetree_items:
             return ''
-        tree_items = self.get_children(tree_alias, None)
-        return tree_items
+        tree_items = self.filter_items(self.get_children(tree_alias, None), 'sitetree')
+        return self.apply_hook(tree_items, 'sitetree')
 
     def children(self, parent_item, navigation_type, use_template, context):
         """Builds and returns site tree item children structure
@@ -381,11 +481,13 @@ class SiteTree(object):
         self.tree_climber(tree_alias, self.get_tree_current_item(tree_alias))
         tree_items = self.get_children(tree_alias, parent_item)
         tree_items = self.filter_items(tree_items, navigation_type)
+        tree_items = self.apply_hook(tree_items, '%s.children' % navigation_type)
         my_template = template.loader.get_template(use_template)
         my_context = template.Context({'sitetree_items': tree_items, 'user': context['user']})
         return my_template.render(my_context)
 
     def get_children(self, tree_alias, item):
+        tree_alias = self.resolve_tree_i18_alias(tree_alias)
         return self.get_cache_entry('parents', tree_alias)[item]
 
     def filter_items(self, items, navigation_type=None):
@@ -393,13 +495,14 @@ class SiteTree(object):
         NB: We do not apply any filters to sitetree in admin app.
 
         """
-        items = copy(items)
+        items_out = copy(items)
         if self.global_context.current_app != 'admin':
             for item in items:
-                if item.hidden == True ^ (not self.check_access(item, self.global_context)) ^ (
-                navigation_type is not None and getattr(item, 'in' + navigation_type, False) != True):
-                    items.remove(item)
-        return items
+                no_access = not self.check_access(item, self.global_context)
+                hidden_for_nav_type = navigation_type is not None and getattr(item, 'in' + navigation_type, False) != True
+                if item.hidden == True or no_access or hidden_for_nav_type:
+                    items_out.remove(item)
+        return items_out
 
     def get_ancestor_item(self, tree_alias, start_from):
         """Climbs up the site tree to resolve root item for chosen one."""
