@@ -1,4 +1,7 @@
+from django.conf import settings
 from django import template
+from django import VERSION
+from django.templatetags.static import PrefixNode
 
 from ..sitetreeapp import SiteTree
 
@@ -7,6 +10,8 @@ register = template.Library()
 # All utility methods are implemented in SiteTree class
 sitetree = SiteTree()
 
+DJANGO_VERSION_INT = int('%s%s%s' % VERSION[:3])
+
 
 @register.tag
 def sitetree_tree(parser, token):
@@ -150,6 +155,15 @@ def sitetree_page_title(parser, token):
         raise template.TemplateSyntaxError, "%r tag requires two arguments. E.g. {%% sitetree_page_title from \"mytree\" %%}." % tokens[0]
 
 
+@register.simple_tag
+def sitetree_admin_img_url_prefix():
+    """Returns static admin images URL prefix. """
+    if DJANGO_VERSION_INT >= 140:
+        return '%sadmin/img' % PrefixNode.handle_simple('STATIC_URL')
+
+    return '%simg/admin' % PrefixNode.handle_simple('ADMIN_MEDIA_PREFIX')
+
+
 class sitetree_treeNode(template.Node):
     """Renders tree items from specified site tree."""
 
@@ -204,7 +218,7 @@ class sitetree_menuNode(template.Node):
 
     def render(self, context):
         tree_items = sitetree.menu(self.tree_alias, self.tree_branches, context)
-        my_context = template.Context({'sitetree_items': tree_items, 'user': context['user']})
+        my_context = template.Context({'sitetree_items': tree_items, 'user': context['user'], 'request': context['request']})
         return self.template.render(my_context)
 
 
